	INCLUDE MKL02.inc		; CPU register definitions
	PRESERVE8
	THUMB				; use thumb instruction set
	
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
I2Cbuf	SPACE 	32			; I2C buffer
SPIbuf	SPACE	32			; SPI buffer
temp	SPACE	4			; temp value
humi	SPACE	4			; humidity value
press	SPACE 	6			; pressure value
volt	SPACE	1			; battery voltage	
turn	SPACE 	1			; 0: display temp/humi; 1: display pressure	
graphP	SPACE	1			; pointer to insert new value to graph
graphC	SPACE	1			; counter to update graph
notch	SPACE	1			; add notch to graph
wake	SPACE	1			; radio wake-up counter	
	SPACE	12			; place holder
graph	SPACE	800			; 8x100 graph area	
battery	SPACE	22			; battery status image	
	EXPORT	I2Cbuf
	EXPORT	SPIbuf	
	EXPORT	temp
	EXPORT	humi
	EXPORT	press
	EXPORT	volt	
	EXPORT	turn		
	EXPORT	graph
	EXPORT	graphP	
	EXPORT	graphC
	EXPORT	notch	
	EXPORT	wake	
	EXPORT	battery

	AREA    |.text|, CODE, READONLY	
MCG_setup	PROC			; set up FEI clock mode
	EXPORT	MCG_setup
	PUTB	0x05, MCG_C1, MCG	; enable FLL clock in STOP mode
mcg1	ldrb	R1, [R0, #MCG_S]	; start checking loop			
	lsrs	R1, #5			; check the IREFST bit to make sure
	bcc	mcg1			;  that FLL switched to external ref

	movs	R3, #0x01		; wait for 1 msec FLL acquisition time
	lsls	R3, #13
	subs	R3, #1
	nop
	bne	.-4
	
	IOORB	0x80, MCG_C4+MCG	; set 24MHz FLL clock frequency 
	PUTW	0x10000, SIM_CLKDIV1, SIM ; set 1:1 core & 1:2 flash/bus clock divider
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
SMC_setup	PROC			; System Mode Controller configuration
	EXPORT	SMC_setup
	PUTB	0x20, SMC_PMPROT, SMC	; allow VLPx modes
	PUTB	0x02, SMC_PMCTRL	; enter VLPS mode in STOP, normal run	
	ldrb	R1, [R0, #SMC_PMSTAT]	; get current mode
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
PORTS_setup	PROC			; I/O Ports configuration
	EXPORT	PORTS_setup
	IOORH	0x0600, SIM_SCGC5+SIM 	; enable clock for I/O ports
	PUTW	0x00B80100, PORT_GPCLR, PORTA	; configure PORTA for GPIO
	PUTW	0x00400103, PORT_GPCLR		; pull-up pin at PA.6 (MISO)
	PUTW	0x00390100, PORT_GPCLR, PORTB
;	PUTW	0x000A0103, PORT_PCR0+4*5 ; enable PB.5 falling edge interrupt	
	PUTW	0x00260103, PORT_GPCLR	; pull-up resistors on input pins PB.1 and PB.2
	PUTW	0x38, GPIO_PDOR, GPIOA	; init port output
	PUTW	0xBF, GPIO_PDDR		; configure PA.6 for input	
	PUTW	0x1E, GPIO_PDOR, GPIOB	; 	
	PUTW	0x19, GPIO_PDDR		; input on CTS, IRQ, INT pins	
;	PUTW	1<<31, 0, NVIC_ISER 	; enable PORTB interrupt in NVIC
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
LPTMR0_setup	PROC			; Low-Power Timer configuration
	EXPORT	LPTMR0_setup
	IOORB	0x01, SIM_SCGC5+SIM 	; enable clock for LPTMR	
	PUTB	0x05, LPTMR_PSR, LPTMR0	; set LPO as clock source, no prescaler	
	PUTW	5000, LPTMR_CMR 	; setup for 5 sec interval
	PUTB	0x41, LPTMR_CSR		; start timer in autoreset, enable interrupt
	PUTW	1<<28, 0, NVIC_ISER 	; enable LPTMR0 interrupt in NVIC	
	bx	LR
	ENDP		
		
;-------------------------------------------------------------------------------
I2C_setup	PROC			; configuration of I2C module as master
	EXPORT	I2C_setup
	PUTW	0x00180300, PORT_GPCLR, PORTA	; connect pins to I2C1	
	PUTW	0x00180200, PORT_GPCLR, PORTB	; connect pins to I2C0		
	IOORB	0xC0, SIM_SCGC4+SIM	; enable clock for both I2C modules
	PUTB	0x14, I2C_F, I2C0	; div by 80 I2C0 clock rate
	PUTB	0x00, I2C_F, I2C1	; div by 20 I2C1 clock rate 
	PUTW	1<<8, 0, NVIC_ISER	; enable I2C0 interrupt in NVIC
	PUTW	1<<9, 0, NVIC_ISER	; enable I2C1 interrupt in NVIC
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
I2C0_transfer	PROC
	EXPORT	I2C0_transfer	
	PUTW	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUTB	0xD0, I2C_C1, I2C0	; enable I2C module TX, and interrupt 
	PUTB	0xF0, I2C_C1		; generate START condition
	adds	R3, #1			; add address count
	mov	R1, R12			; send slave address
	strb	R1, [R0, #I2C_D]
	wfi				; enter sleep mode	
        PUTB	0, I2C_C1		; I2C reset
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
Send_LCD	PROC
	EXPORT	Send_LCD
	push	{R4}	
	movs	R4, #0			; no read	
	PUTW	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUTB	0xD0, I2C_C1, I2C1	; enable I2C module TX, and interrupt 
	PUTB	0xF0, I2C_C1		; generate START condition
	adds	R3, #1			; add address count
	mov	R1, R12			; send slave address
	strb	R1, [R0, #I2C_D]
	wfi				; enter sleep mode	
        PUTB	0, I2C_C1		; I2C reset
	pop	{R4}
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
SPI0_setup	PROC
	EXPORT	SPI0_setup
	IOORW	0x400000, SIM_SCGC4+SIM	; enable clock for SPI module	
	PUTB	0x21, SPI_BR, SPI0	; set bus_freq/12 (1 MHz) baud rate
	PUTW	1<<10, 0, NVIC_ISER	; enable SPI0 interrupt in NVIC
	ldr	R6, =GPIOA
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
RAM_setup	PROC			; init some RAM values
	EXPORT	RAM_setup
	PUTB	0, 0, turn		; turn = 0
	PUTW	0, 0, temp		; temp = 0
	PUTW	0, 0, humi		; humi = 0
	PUTW	0, 0, press		; press = 0
	PUTH	0, 4
	PUTB	36, 0, volt		; voltage = 3.6v
	PUTB	3, 0, notch		; force notch at first pressure measurement
	PUTB	0, 0, wake
	bx	LR
	ENDP
		
	ALIGN
	END
	