	INCLUDE MKL02.inc		; CPU register definitions
	PRESERVE8
	THUMB				; use thumb instruction set
		
	IMPORT	I2Cbuf	
	IMPORT	Send_LCD	
	IMPORT	temp
	IMPORT	humi
	IMPORT	press
	IMPORT	graph	
	IMPORT	graphP
	IMPORT	graphC
	IMPORT	notch	
	IMPORT	battery	
	IMPORT	volt	
	IMPORT	Get_Pressure		

graphW	EQU	100			; graph area width (100 pixels)
graphH	EQU	8			; graph area height (8 pages)
graphO	EQU	4			; graph offset (4 pixels)
graphR	EQU	90			; rate to update pressure graph	
presMin	EQU	722 			; min pressure (mmHg)
	
	AREA    |.text|, CODE, READONLY
;-------------------------------------------------------------------
Display_Temp_Humi	PROC
	EXPORT	Display_Temp_Humi
	push	{LR}
	bl	Clear_LCD
	ldr	R7, =temp
	ldrb	R1, [R7, #2]		; get raw temp value
	sxtb	R1, R1			; sign extend it to word
	ldr	R7, =Sign
	cmp	R1, #0
	bge	. + 4			; jump over the next instr.
	adds	R7, #34			; pointer to '-'

	movs	R4, #118		; column #
	movs	R5, #4			; page #
	bl	Display_Bitmap		; sign ('-' if negative, blank otherwise)

	ldr	R7, =temp
	ldrb	R0, [R7, #0]		; R0 = tens of temp
	movs	R2, #0xFF
	adds	R2, #33			; R2 = digit size in font
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 1st digit address
	movs	R4, #137		; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display tens of temp

	ldr	R7, =temp
	ldrb	R0, [R7, #1]
	movs	R2, #0xFF
	adds	R2, #33
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 2nd digit address
	movs	R4, #175		; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display units of temp	

	movs	R4, #212		; column #
	movs	R5, #7			; page #			
	ldr	R7, =Deg_C		; 'C' symbol pointer
	bl	Display_Bitmap	

	ldr	R7, =humi
	ldrb	R0, [R7, #0]		; R0 = tens of humi
	movs	R2, #0xFF
	adds	R2, #33
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 1st digit address
	movs	R4, #0			; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display tens of humi

	ldr	R7, =humi
	ldrb	R0, [R7, #1]
	movs	R2, #0xFF
	adds	R2, #33
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 2nd digit address
	movs	R4, #38			; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display units of humi
	
	movs	R4, #76			; column #
	movs	R5, #7			; page #
	ldr	R7, =Percent		; '%' symbol pointer
	bl	Display_Bitmap	
	
	movs	R4, #105		; column #
	movs	R5, #0			; page #
	ldr	R7, =battery		; TX battery status
	bl	Display_Bitmap	
	pop	{PC}
	ENDP

;-------------------------------------------------------------------
Draw_Graph	PROC
	EXPORT	Draw_Graph
	push	{LR}
	movs	R2, #0x01
	rev16	R2, R2
	adds	R2, #(graphH-1)
	bl	Set_Cursor		; (x,y) = (1,7)
	movs	R3, #(graphO-1)		; frame width
	ldr	R7, =I2Cbuf
	movs	R1, #0x80
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update		
	bl	Send_LCD		; draw top line	
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update
	
	movs	R2, #(graphW+graphO)
	rev16	R2, R2
	adds	R2, #(graphH-1)
	bl	Set_Cursor		; (x,y) = (105,7)
	movs	R3, #(graphO-1)		; frame width
	ldr	R7, =I2Cbuf
	movs	R1, #0x80
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update		
	bl	Send_LCD		; draw top line	
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update

Bottom_frame
	movs	R2, #1
	rev16	R2, R2
	bl	Set_Cursor		; (x,y) = (1,0)
	movs	R3, #(graphO-1)		; part of frame width	
	ldr	R7, =I2Cbuf
	movs	R1, #0x01
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update	
	bl	Send_LCD		; draw bottom line	
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update

	movs	R2, #(graphW+graphO)
	rev16	R2, R2
	bl	Set_Cursor		; (x,y) = (105,0)
	movs	R3, #(graphO-1)		; part of frame width	
	ldr	R7, =I2Cbuf
	movs	R1, #0x01
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update	
	bl	Send_LCD		; draw bottom line	
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update

Left_Frame
	movs	R5, #(graphH-1)
dg1	mov	R2, R5	
	bl	Set_Cursor		; (x,y) = (0, y)
	movs	R3, #1		
	ldr	R7, =I2Cbuf
	movs	R1, #0xFF
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update
	bl	Send_LCD		; left line
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update	
	subs	R5, #1
	bge	dg1

Right_Frame
	movs	R5, #(graphH-1)
dg2	movs	R2, #(graphW+2*graphO-1)
	rev16	R2, R2
	adds	R2, R5			
	bl	Set_Cursor		; (x,y) = (105, y)
	movs	R3, #1		
	ldr	R7, =I2Cbuf
	movs	R1, #0xFF
	strb	R1, [R7]
	add	R10, R11		; disable I2Cbuf address update	
	bl	Send_LCD		; right line	
	movs	R1, #1
	mov	R10, R1			; enable I2Cbuf address update	
	subs	R5, #1
	bge	dg2

	ldr	R7, =graphC		; time to update pressure graph?
	ldrb	R0, [R7]
	cmp	R0, #graphR
	bne	dg0

	ldr	R7, =graphP		
	ldrb	R0, [R7]		
	adds	R0, #1			; increment graphP modulo graphW
	cmp	R0, #graphW
	bne	. + 4			; jump over the next instr
	movs	R0, #0
	strb	R0, [R7]

	ldr	R7, =notch		; update the notch counter
	ldrb	R1, [R7]
	adds	R1, #1
	movs	R0, #0x03
	ands	R1, R0
	strb	R1, [R7]
	
	ldr	R7, =press
	ldrh	R1, [R7, #4]		; get pressure value
	movs	R2, #(presMin/256)
	rev16	R2, R2
	adds	R2, #(presMin % 256)	; R2 = min pressure on graph
	subs	R1, R2			; R1 = press - presMin
	adds	R1, #graphO
	bpl	dg3
	movs	R1, #0			; saturate R1 with 0
dg3	ldr	R7, =graphP
	ldrb	R2, [R7]		; R2 = column to put new pressure
	ldr	R7, =graph
	adds	R7, R2			; R7 = address of top column part

	movs	R5, #64			; top value in column
dg4	mov	R4, R5
	movs	R3, #0xFF		; init value to put on graph
	subs	R4, R1			; R4 = amount of shift
	ble	dg6 
	cmp	R4, #8
	blt	dg5
	movs	R4, #8	
dg5	lsrs	R3, R4			; R3 = final value
dg6	cmp	R5, #64			; special processing of the top row
	bne	dg7
	movs	R0, #0x0F
	ands	R3, R0	
	adds	R3, #0x80
	ldr	R0, =notch
	ldrb	R0, [R0]
	orrs	R0, R0
	bne	dg7
	adds	R3, #0x60		; add a notch
dg7	cmp	R5, #8			; special processing of the bottom row
	bne	dg8
	movs	R0, #0xF0
	ands	R3, R0
	adds	R3, #1
	ldr	R0, =notch
	ldrb	R0, [R0]
	orrs	R0, R0
	bne	dg8
	adds	R3, #0x06		; add a notch	
dg8	strb	R3, [R7]
	adds	R7, #graphW
	subs	R5, #8
	bne	dg4

dg0	movs	R5, #7			; R5 = # of rows-1
	movs	R0, #0			 
	mov	R8, R0			; R8 = to add to R7
	movs	R0, #graphW
	mov	R9, R0			; R9 = update for R8
	ldr	R7, =graphP		
	ldrb	R4, [R7]		; R4 = column to update in circ buffer
	adds	R4, #1	
dg9	movs	R2, #graphO		; R2 = cursor X
	rev16	R2, R2
	adds	R2, R5			; R2 = 0 0 4 y
	push	{R4}
	bl	Set_Cursor
	pop	{R4}	
	
	movs	R3, #graphW
	subs	R3, R4			; R3 = transfer amount
	beq	dg10
	ldr	R7, =graph
	adds	R7, R4			; R7 = starting data address in circ buffer
	add	R7, R8
	push	{R4}
	bl	Send_LCD
	pop	{R4}

dg10	ldr	R7, =graph
	add	R7, R8
	mov	R3, R4
	push	{R4}
	bl	Send_LCD
	pop	{R4}

	add	R8, R9
	subs	R5, #1	
	cmp	R5, R11
	bne	dg9
	pop	{PC}
	ENDP
	
;-------------------------------------------------------------------
Display_Pressure PROC
	EXPORT	Display_Pressure
	push	{LR}	
	ldr	R7, =graphC		; time to request new pressure?
	ldrb	R0, [R7]
	subs	R0, #1
	bne	. + 4			; jump over the next instr
	movs	R0, #graphR	
	strb	R0, [R7]
	cmp	R0, #graphR
	bne	dp1
	bl	Get_Pressure	
	
dp1	bl	Clear_LCD
	bl	Draw_Graph	
	ldr	R7, =press
	movs	R2, #0xFF
	adds	R2, #33			; R2 = digit font size
	ldrb	R0, [R7, #0]		; R0 = hundreds of pressure
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 1st digit address
	movs	R4, #128		; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display tens of humi

	ldr	R7, =press
	ldrb	R0, [R7, #1]
	movs	R2, #0xFF
	adds	R2, #33
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 2nd digit address
	movs	R4, #166		; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display tens of press

	ldr	R7, =press
	ldrb	R0, [R7, #2]
	movs	R2, #0xFF
	adds	R2, #33
	muls	R0, R2, R0		; R0 = digit offset
	ldr	R7, =Font
	adds	R7, R0			; R7 = 3rd digit address
	movs	R4, #204		; x-coord
	movs	R5, #7			; y-coord
	bl	Display_Digit		; display units of press
	pop	{PC}
	ENDP	
		
;---------------------------------------------------------------------
Display_Digit	PROC			; R7=bitmap offset, R4=x, R5=y
	EXPORT	Display_Digit
	push	{LR}	
	movs	R0, #36			; font width
	mov	R8, R0
	movs	R0, #7			; font height (-1)
	mov	R9, R0
	b	dbm1

Display_Bitmap				; R7=bitmap offset, R4=x, R5=y
	EXPORT	Display_Bitmap
	push	{LR}	
	ldrb	R0, [R7]		; get bitmap width
	mov	R8, R0
	ldrb	R0, [R7, #1]		; get bitmap height (in pages)
	subs	R0, #1
	mov	R9, R0
	adds	R7, #2

dbm1	push	{R4}
	mov	R2, R4			; R2: 0 0 0 x
	rev16	R2, R2			; R2: 0 0 x 0
	add 	R2, R5			; R2: 0 0 x y
	push	{R7}			; save bitmap pointer
	bl	Set_Cursor		; set page & column
	pop	{R7}			; restore bitmap pointer
	mov	R3, R8			; bitmap width
	bl	Send_LCD		; send bitmap row
	pop	{R4}
	subs	R5, #1
	add	R9, R11			; loop over LCD pages	
	cmp	R9, R11
	bne	dbm1
	pop	{PC}
	ENDP		

;---------------------------------------------------------------------
LCD_setup	PROC			
	EXPORT	LCD_setup
	push	{LR}
	movs	R1, #1
	mov	R10, R1			; R10 = 1
	subs	R1, #2
	mov	R11, R1			; R11 = -1	
	movs	R1, #0x70		; slave address
	mov	R12, R1
	
	movs	R1, #1
	lsls	R1, #22
puc1	subs	R1, #1			; power-up delay
	nop
	bne	puc1
	ldr	R7, =LCD_INIT		; load init sequence
	movs	R3, #16
	bl	Send_LCD
	
	movs	R1, #0x1
	lsls	R1, #10
puc2	subs	R1, #1
	nop
	bne	puc2
	bl	Clear_LCD
	
	ldr	R7, =graph		; init graph area
	movs	R2, #graphW
	movs	R3, #0x80
cga1	strb	R3, [R7]		; init top row	
	adds	R7, #1
	subs	R2, #1
	bne	cga1
	
	movs	R1, #graphW
	movs	R2, #(graphH-2)
	muls	R2, R1, R2		; R2 = graph area size in bytes
	movs	R1, #0x00
cga2	strb	R1, [R7]
	adds	R7, #1
	subs	R2, #1
	bne	cga2

	movs	R2, #graphW
	movs	R3, #0x01
cga3	strb	R3, [R7]		; init bottom row	
	adds	R7, #1
	subs	R2, #1
	bne	cga3
	
	movs	R1, #graphW		; set up graph pointer
	subs	R1, #1			; new pressure value must be output
	ldr	R7, =graphP		; here and the circ. buffer should	
	strb	R1, [R7]		; be dumped starting w. next col. value	
	movs	R1, #1			; make sure to upgate pressure graph now
	strb	R1, [R7, #1]		; by setting graphC to 1	
	
	ldr	R7, =battery		; create battery image in RAM
	movs	R1, #20
	strb	R1, [R7, #0]
	movs	R1, #1
	strb	R1, [R7, #1]
	adds	R7, #2
	movs	R1, #0xFF
	movs	R2, #18
cga4	strb	R1, [R7]
	adds	R7, #1
	subs	R2, #1
	bne	cga4
	movs	R1, #0x3C
	strb	R1, [R7, #0]
	strb	R1, [R7, #1]

	pop	{PC}
	ENDP
	ALIGN		
	LTORG
LCD_INIT
	DCB	0xF1, 0x3F, 0xF2, 0x00, 0xF3, 0x3F, 0x81, 0xB7, 0xC0, 0x04, 0xA3, 0xE9, 0xA9, 0xD1, 0x89, 0xC4
	ALIGN

;---------------------------------------------------------------------
Clear_LCD	PROC
	EXPORT	Clear_LCD
	push	{LR}	
	movs	R2, #0
	bl	Set_Cursor

	movs	R3, #0x78
	lsls	R3, #4			; R3 = 240*8
	ldr	R7, =I2Cbuf
	movs	R1, #0
	strb	R1, [R7]
	mov	R10, R1			; disable I2Cbuf address update
	bl	Send_LCD
	movs	R1, #1			; re-enable address update
	mov	R10, R1	
	pop	{PC}			
	ENDP
	ALIGN	
	LTORG
		
;---------------------------------------------------------------------	
Set_Cursor	PROC			; R2: 0 0 x y
 	push	{LR}
	ldr	R7, =I2Cbuf
	movs	R1, #0x70		; slave address
	mov	R12, R1			; for commands
	
	mov	R1, R2
	movs	R3, #0x0F
	ands	R1, R3
	adds	R1, #0x60
	strb	R1, [R7, #0]
	mov	R1, R2
	lsrs	R1, #4
	ands	R1, R3
	adds	R1, #0x70
	strb	R1, [R7, #1]	

	rev16	R2, R2
	mov	R1, R2
	ands	R1, R3
	strb	R1, [R7, #2]
	mov	R1, R2
	lsrs	R1, #4
	ands	R1, R3
	adds	R1, #0x10
	strb	R1, [R7, #3]
	
	movs	R3, #4			; send 4 bytes
	bl	Send_LCD
	movs	R1, #0x72		; restore slave address
	mov	R12, R1			; for data
	pop	{PC}
	ENDP
	ALIGN	
	LTORG
	
;====================================================================		
	ALIGN
	INCLUDE	font.inc
	ALIGN
	END
