	INCLUDE MKL02.inc		; CPU register definitions
	PRESERVE8
	THUMB				; use thumb instruction set
		
	IMPORT	I2Cbuf	
	IMPORT	I2C0_transfer	
	IMPORT	press		
		
	AREA    |.text|, CODE, READONLY	
LPS25H_setup	PROC
	EXPORT	LPS25H_setup
	push	{LR}	
	movs	R1, #1
	mov	R10, R1
	
	movs	R1, #0xB8		; sensor address+write
	mov	R12, R1
	ldr	R7, =I2Cbuf
	movs	R1, #0x20		; CTRL_REG1 address
	strb	R1, [R7, #0]
	movs	R1, #0x80		; power-up + one-shot
	strb	R1, [R7, #1]
	movs	R3, #2			; send 2 bytes
	movs	R4, #0
	bl	I2C0_transfer
	
	ldr	R7, =I2Cbuf
	movs	R1, #0xB9		; RPDS_L register+write
	strb	R1, [R7, #0]
	movs	R1, #0xE0		; -18 hpa (correction)	
	strb	R1, [R7, #1]
	movs	R1, #0xFE
	strb	R1, [R7, #2]
	movs	R3, #3			; send 3 bytes
	movs	R4, #0
	bl	I2C0_transfer
	movs	R1, #0x72		; restore slave address
	mov	R12, R1			; for LCD data	
	pop	{PC}
	ENDP
		
;-------------------------------------------------------------------	
Request_Pressure PROC
	EXPORT	Request_Pressure
	push	{LR}
	movs	R1, #0xB8		; sensor address+write
	mov	R12, R1
	ldr	R7, =I2Cbuf
	movs	R1, #0x21		; CTRL_REG2 register+write
	strb	R1, [R7, #0]
	movs	R1, #0x01
	strb	R1, [R7, #1]
	movs	R3, #2			; send 2 bytes
	movs	R4, #0
	bl	I2C0_transfer
	movs	R1, #0x72		; restore slave address
	mov	R12, R1			; for LCD data	
	pop	{PC}
	ENDP
	
;-------------------------------------------------------------------
Read_Pressure	PROC
	EXPORT	Read_Pressure
	push	{LR}	
	movs	R1, #0xB8		; sensor address+write
	mov	R12, R1		
	ldr	R7, =I2Cbuf
	movs	R1, #0xA8		; PRESS_OUT_XL register
	strb	R1, [R7, #0]
	movs	R3, #1			; send 1 bytes
	movs	R4, #0		
	bl	I2C0_transfer
	
	movs	R1, #0xB9		; sensor address+write
	mov	R12, R1		
	ldr	R7, =I2Cbuf
	movs	R3, #0			; send 0 bytes
	movs	R4, #3			; receive 3 bytes
	bl	I2C0_transfer	
	movs	R1, #0x72		; restore slave address
	mov	R12, R1			; for LCD data
	
	ldr	R7, =I2Cbuf
	movs	R1, #0			; make sure that LSB of press
	strb	R1, [R7, #3]		; is 0
	ldr	R1, [R7]		; R1 = pressure
	lsrs	R1, #12			; R1 = pressure/4096
	movs	R0, #3
	muls	R1, R0, R1	
	lsrs	R1, #2			; R1 = pressure in mmHg
	ldr	R0, =press
	strh	R1, [R0, #4]		; save press in RAM
	pop	{PC}
	ENDP

;-------------------------------------------------------------------
Get_Pressure	PROC
	EXPORT	Get_Pressure
	push	{LR}	
	bl	Request_Pressure
	PUTB	0x80, LPTMR_CSR, LPTMR0	; stop timer
	ldr	R1, [R0, #LPTMR_CMR]	; get old timer value
	push	{R1}			; save it
	PUTW	60, LPTMR_CMR  		; setup for 60 msec interval
	PUTB	0x41, LPTMR_CSR		; start timer in autoreset, enable interrupt
	wfi
	PUTB	0x80, LPTMR_CSR, LPTMR0	; stop timer
	pop	{R1}			; restore interval value
	str	R1, [R0, #LPTMR_CMR]  
	PUTB	0x41, LPTMR_CSR		; start timer in autoreset, enable interrupt
	bl	Read_Pressure
	
Process_Pressure			; convert pressure into BCD
	ldr	R7, =press
	ldrh	R1, [R7, #4]		; R1 = pressure value in mmHg	
	movs	R0, #0x02		; x/100 = x*656 >> 16
	rev16	R0, R0			; works for x<1000
	adds	R0, #0x90		; 656_10 = 290_16
	muls	R0, R1, R0
	lsrs	R0, #16			; R0 = R1/100
	strb	R0, [R7, #0]		; save hundreds of press in RAM
	movs	R2, #100
	muls	R2, R0, R2
	subs	R1, R2			; R1 = R1%100

	movs	R0, #103
	muls	R0, R1, R0
	lsrs	R0, #10			; R0 = R1/10
	movs	R2, #10
	muls	R2, R0, R2
	subs	R1, R2			; R1 = R1%10
	strb	R0, [R7, #1]		; save tens of press in RAM
	strb	R1, [R7, #2]		; save units of press in RAM	
	pop	{PC}
	ENDP
		
	ALIGN
	END