	INCLUDE MKL02.inc		; CPU register definitions
	EXPORT	start
	
	IMPORT	MCG_setup
	IMPORT	SMC_setup
	IMPORT	PORTS_setup
	IMPORT	LPTMR0_setup	
	IMPORT	I2C_setup
	IMPORT	SPI0_setup
	IMPORT	LCD_setup
	IMPORT	LPS25H_setup
	IMPORT	SI4362_setup
	IMPORT	RAM_setup		
	IMPORT	Start_RX		
	IMPORT	Get_Pressure
	IMPORT	Display_Pressure
	IMPORT	Get_Temp_Humi		
	IMPORT	Display_Temp_Humi
	IMPORT	turn	
	IMPORT	wake	
IRQ	EQU	1+1			; IRQ =	PB.1

	PRESERVE8
	THUMB				; use thumb instruction set
	AREA    |.text|, CODE, READONLY
;----------------------------------------------------------------------------
start	PROC				; main user code
	bl	MCG_setup		; configure clock for VLPR mode
	bl	SMC_setup		; enable clocks to peripherals
	bl	PORTS_setup		; I/O ports setup
	bl	I2C_setup		; setup I2C module
	bl	LCD_setup		; configure LCD driver circuir
	bl	LPS25H_setup		; configure LPS25H sensor
	bl	SPI0_setup		; setup SPI module	
	bl	SI4362_setup		; configure radio
	bl	Start_RX		; start receiving data
	bl	RAM_setup		; set RAM variables	
	bl	LPTMR0_setup		; setup timer for display update

loop					; MAIN LOOP
	PUTW	SLEEPDEEP, 0, SCB_SCR 	; enable VLPS/LSS sleep mode
	wfi				; enter deep sleep mode
	nop				; wake-up by LPTMR0
	
	ldr	R0, =GPIOB
	ldrb	R1, [R0, #GPIO_PDIR]	; check for radio IRQ
	lsrs	R1, #IRQ		; new package received?	
	bcs	l1			; NO  - keep waiting
	
l0	bl	Get_Temp_Humi		; YES - get new received data
	ldr	R0, =wake		; update radio wake-up counter 
	movs	R1, #10			; # of 5-sec intervals to sleep
	strb	R1, [R0]		; reload wake-up counter
	b	l2			; radio is put on sleep now

l1	ldr	R0, =wake		; update radio wake-up counter 
	ldrb	R1, [R0]
	subs	R1, #1
	strb	R1, [R0]		; sleep interval expired?	
	bne	l2			; NO  - keep waiting
	bl	Start_RX		; YES - wake-up radio 	

l2	ldr	R0, =turn		; display data on LCD
	ldrb	R1, [R0]
	movs	R2, #1
	subs	R1, R2, R1		; flip the 0/1 turn variable
	strb	R1, [R0]
	bne	l3					
	bl	Display_Temp_Humi	; turn=0: display temperature/humidity/battery 
	b	loop

l3	bl	Display_Pressure	; turn=1: display pressure with graph	
	b	loop
	ENDP
	
	ALIGN 
	END
	