	INCLUDE MKL02.inc		; CPU register definitions
	IMPORT	SPIbuf
	IMPORT	temp
	IMPORT	humi
	IMPORT	volt
	IMPORT	battery	
	
CS	EQU	1<<5			; CS  = PA.A (chip select)
CTS	EQU	2+1			; CTS = PB.2 (clear to send)
	
	PRESERVE8
	THUMB				; use thumb instruction set
	AREA    |.text|, CODE, READONLY

;-----------------------------------------------------------------
SI4362_setup	PROC
	EXPORT	SI4362_setup
	push	{LR}	
	ldr	R7, =SI4362_Config
	ldrb	R2, [R7]		; get # of commands to send
	adds	R7, #1
s4s	push	{R2}
	bl	Send_SI4362
	pop	{R2}
	subs	R2, #1
	bne	s4s
	pop	{PC}
	ENDP
		
;-----------------------------------------------------------------
Send_SI4362	PROC			; send command/data/property to Si4012
	EXPORT	Send_SI4362	
	movs	R4, #0			; # of bytes to RX		
Read_SI4362				; R4 is set by caller
	EXPORT	Read_SI4362
	ldr	R0, =GPIOB	
wcts	ldrh	R1, [R0, #GPIO_PDIR]	
	lsrs	R1, #CTS		; check CTS flag		
	bcc	wcts			; wait for CTS
	
	ldrb	R3, [R7]		; R3 = #bytes to TX
	adds	R7, #1	
	PUTW	0x00C00300, PORT_GPCLR, PORTA	; connect pins to SPI 
	PUTW	0x00010300, PORT_GPCLR, PORTB		
	PUTW	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUTB	0x50, SPI_C1, SPI0	; enable SPI module TX, and interrupt
	movs	R1, #CS
	str	R1, [R6, #GPIO_PCOR]	; CS down
	nop
	PUTB	0x70, SPI_C1		; enable SPI TX empty interrupt	
	wfi	
	movs	R1, #CS
	str	R1, [R6, #GPIO_PSOR]	; CS up
	PUTB	0, SPI_C1		; SPI reset
	PUTW	0x00C00103, PORT_GPCLR, PORTA	; disconnect pins from SPI 
	PUTW	0x00010100, PORT_GPCLR, PORTB	
	bx	LR		
	ENDP

;-----------------------------------------------------------------------
; package structure: length(1) ID(0xAA) humi(1) temp(1) batt*10(1)
Get_Temp_Humi	PROC
	EXPORT	Get_Temp_Humi
	push	{LR}	
	ldr	R7, =FIFO_INFO		; FIFO_INFO command			
	bl	Send_SI4362
	ldr	R7, =READ_CMD_BUFF	; READ_CMD_BUFF command
	movs	R4, #0x03		; response length
	bl	Read_SI4362		; get response
	subs	R7, #2	
	ldrb	R4, [R7]		; get RX_FIFO_COUNT

	cmp	R4, #0
	beq	srx			; do not attempt to read 0 length buffer
	ldr	R7, =READ_RX_FIFO	; READ_RX_FIFO command
	bl	Read_SI4362		; load package from FIFO
	subs	R7, #4
	ldrb	R1, [R7, #0]		; get TX ID
	cmp	R1, #0xAA		; ID match?	
	bne	srx			; NO - ignore received data
	
	ldr	R0, =temp
	ldrb	R1, [R7, #2]		; get received temp
	strb	R1, [R0, #2]		; save temp in RAM
	ldrb	R1, [R7, #1]		; get received humi
	strb	R1, [R0, #6]		; save humi in RAM
	ldrb	R1, [R7, #3]		; get battery voltage
	cmp	R1, #0x10
	ble	gth1
	strb	R1, [R0, #14]		; save battery voltage in RAM
	
gth1	ldr	R7, =FRR_A_READ		; FRR_A_READ command
	movs	R4, #0x04		; read all 4 FRRs
	bl	Read_SI4362		; get RSSI and PH/Modem/Chip stati
	subs	R7, #3
	ldrb	R1, [R7]		; get PH status bits
	lsrs	R1, #4			; check for CRC bit		
	bcs	srx			; ignore received data for CRC mismatch

	ldr	R7, =CLEAR_FIFO		; valid package received, go to sleep			
	bl	Send_SI4362		; CLEAR_FIFO command
	ldr	R7, =CLEAR_IRQ		; clear interrupts
	bl	Send_SI4362
	ldr	R7, =MOVE_TO_SLEEP	; move to SLEEP state
	bl	Send_SI4362

Process_Temp	
	ldr	R7, =temp
	ldrb	R1, [R7, #2]		; get raw temp value
	sxtb	R1, R1			; sign extend it to word
	cmp	R1, #0
	bge	. + 4			; jump over the next instruction	
	rsbs	R1, R1, #0		; negate temp if negative

	movs	R0, #103		; process temp
	muls	R0, R1, R0
	lsrs	R0, #10			; R0 = R1/10
	movs	R2, #10
	muls	R2, R0, R2		; R2 = [R1/10]*10
	bne	.+4
	movs	R0, #10			; do not display leading 0
	subs	R1, R2			; R1 = R1%10
	strb	R0, [R7, #0]		; save tens  of temp in RAM
	strb	R1, [R7, #1]		; save units of temp in RAM

Process_Humi
	ldr	R7, =humi
	ldrb	R1, [R7, #2]		; get raw humi value
	sxtb	R1, R1			; sign extend it to word
	cmp	R1, #0
	bge	. + 4			; jump over the next instruction	
	movs	R1, #0			; truncate humi to 0 if negative
	cmp	R1, #99
	ble	. + 4			; make sure that humi is in [0, 99]
	movs	R1, #99

	subs	R1, #2			; sensor correction
	movs	R0, #103		; process humi
	muls	R0, R1, R0
	lsrs	R0, #10			; R0 = R1/10
	movs	R2, #10
	muls	R2, R0, R2		; R2 = [R1/10]*10
	bne	.+4
	movs	R0, #10			; do not display leading 0
	subs	R1, R2			; R1 = R1%10
	strb	R0, [R7, #0]		; save tens  of humi in RAM
	strb	R1, [R7, #1]		; save units of humi in RAM	

Process_Battery
	ldr	R7, =volt
	ldrb	R2, [R7]		; R2 = battery voltage * 10
	movs	R3, #16
	subs	R2, #20			; R2 = # of full lines
	subs	R3, R3, R2		; R3 = # of empty lines
	movs	R1, #0xFF		; full pattern
	movs	R0, #0x81		; empty pattern
	ldr	R7, =(battery+3)
pbt1	orrs	R2, R2
	ble	pbt2
	strb	R1, [R7]
	adds	R7, #1
	subs	R2, #1
	bgt	pbt1
	
pbt2	orrs	R3, R3
	ble	srx
	strb	R0, [R7]
	adds	R7, #1
	subs	R3, #1
	bgt	pbt2
	pop	{PC}

srx	bl	Start_RX
	pop	{PC}
	ENDP
		
Start_RX	PROC			; Enter RX mode
	EXPORT	Start_RX
	push	{LR}
	ldr	R7, =CLEAR_FIFO		; CLEAR_FIFO command						
	bl	Send_SI4362	
	
	ldr	R7, =MOVE_TO_RX		; move to RX state
	bl	Send_SI4362	
		
	ldr	R7, =START_RX		; start RX	
	bl	Send_SI4362		; stay in READY if invalid

sr1	ldr	R7, =REQUEST_DEVICE_STATE		
	bl	Send_SI4362
	ldr	R7, =READ_CMD_BUFF	; READ_CMD_BUFF command	
	movs	R4, #0x03		; response length
	bl	Read_SI4362		; get response	
	subs	R7, #2
	ldrb	R1, [R7]		; R1 = state
	cmp	R1, #0x08		; wait for RX state
	bne	sr1

	ldr	R7, =CLEAR_IRQ		; clear interrupts
	bl	Send_SI4362
	pop	{PC}
	ENDP
		
;--- RADIO COMMANDS -----------------------------------------------------
FIFO_INFO
	DCB	0x02, 0x15, 0x00
CLEAR_FIFO
	DCB	0x02, 0x15, 0x02
READ_CMD_BUFF
	DCB	0x01, 0x44
READ_RX_FIFO
	DCB	0x01, 0x77
FRR_A_READ
	DCB	0x01, 0x50
CLEAR_IRQ
	DCB	0x01, 0x20
MOVE_TO_SLEEP
	DCB	0x02, 0x34, 0x01
MOVE_TO_RX
	DCB	0x02, 0x34, 0x08
START_RX
	DCB	0x08, 0x32, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x08
REQUEST_DEVICE_STATE
	DCB	0x01, 0x33

;---------------------------------------------------------------------
SI4362_Config
	DCB     0x1F    ; config data size
        DCB     0x07, 0x02, 0x01, 0x00, 0x01, 0xC9, 0xC3, 0x80
        DCB     0x08, 0x13, 0x00, 0x08, 0x00, 0x00, 0x27, 0x00, 0x00
        DCB     0x06, 0x11, 0x00, 0x02, 0x00, 0x52, 0x00
        DCB     0x05, 0x11, 0x00, 0x01, 0x03, 0x60
        DCB     0x06, 0x11, 0x01, 0x02, 0x00, 0x01, 0x18
        DCB     0x08, 0x11, 0x02, 0x04, 0x00, 0x0A, 0x04, 0x06, 0x08
        DCB     0x0D, 0x11, 0x10, 0x09, 0x00, 0x04, 0x14, 0x00, 0x0F, 0x31, 0x00, 0x00, 0x00, 0x00
        DCB     0x09, 0x11, 0x11, 0x05, 0x00, 0x01, 0xCB, 0x89, 0x00, 0x00
        DCB     0x0B, 0x11, 0x12, 0x07, 0x00, 0x84, 0x01, 0x08, 0xFF, 0xFF, 0x00, 0x02
        DCB     0x07, 0x11, 0x12, 0x03, 0x08, 0x2A, 0x01, 0x00
        DCB     0x10, 0x11, 0x12, 0x0C, 0x0C, 0x30, 0x00, 0x01, 0x04, 0x82, 0x00, 0x04, 0x00, 0x0A, 0x00, 0x00, 0x00
        DCB     0x10, 0x11, 0x12, 0x0C, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        DCB     0x10, 0x11, 0x12, 0x0C, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        DCB     0x09, 0x11, 0x12, 0x05, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00
        DCB     0x10, 0x11, 0x20, 0x0C, 0x00, 0x02, 0x00, 0x07, 0x00, 0x2E, 0xE0, 0x01, 0xC9, 0xC3, 0x80, 0x00, 0x00
        DCB     0x05, 0x11, 0x20, 0x01, 0x0C, 0xB4
        DCB     0x0B, 0x11, 0x20, 0x07, 0x19, 0x80, 0x08, 0x03, 0xC0, 0x00, 0x24, 0x20
        DCB     0x0D, 0x11, 0x20, 0x09, 0x22, 0x03, 0x0D, 0x00, 0xA7, 0xC6, 0x00, 0x54, 0x02, 0xC2
        DCB     0x0B, 0x11, 0x20, 0x07, 0x2C, 0x04, 0x36, 0x80, 0x01, 0x5D, 0x07, 0x80
        DCB     0x05, 0x11, 0x20, 0x01, 0x35, 0xE2
        DCB     0x0D, 0x11, 0x20, 0x09, 0x38, 0x11, 0xAB, 0xAB, 0x00, 0x02, 0xFF, 0xFF, 0x00, 0x2B
        DCB     0x0D, 0x11, 0x20, 0x09, 0x42, 0xA4, 0x02, 0xD6, 0x81, 0x01, 0x29, 0x01, 0x80, 0xFF
        DCB     0x05, 0x11, 0x20, 0x01, 0x4C, 0x01
        DCB     0x05, 0x11, 0x20, 0x01, 0x4E, 0x40
        DCB     0x05, 0x11, 0x20, 0x01, 0x51, 0x08	
        DCB     0x10, 0x11, 0x21, 0x0C, 0x00, 0xA2, 0x81, 0x26, 0xAF, 0x3F, 0xEE, 0xC8, 0xC7, 0xDB, 0xF2, 0x02, 0x08
        DCB     0x10, 0x11, 0x21, 0x0C, 0x0C, 0x07, 0x03, 0x15, 0xFC, 0x0F, 0x00, 0xA2, 0x81, 0x26, 0xAF, 0x3F, 0xEE
        DCB     0x10, 0x11, 0x21, 0x0C, 0x18, 0xC8, 0xC7, 0xDB, 0xF2, 0x02, 0x08, 0x07, 0x03, 0x15, 0xFC, 0x0F, 0x00
        DCB     0x0B, 0x11, 0x23, 0x07, 0x00, 0x2C, 0x0E, 0x0B, 0x04, 0x0C, 0x73, 0x03
        DCB     0x10, 0x11, 0x30, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        DCB     0x0C, 0x11, 0x40, 0x08, 0x00, 0x3B, 0x09, 0x99, 0x99, 0x22, 0x22, 0x20, 0xFF
	ALIGN
	END	