	INCLUDE MKL02.inc		; CPU register definitions
	AREA	STACK, NOINIT, READWRITE, ALIGN=3	
stmem	SPACE	0x100			; stack size
top_of_stack		

	AREA    |.ARM.__at_0x400|, CODE, READONLY
	DCB     0xFF, 0xFF, 0xFF, 0xFF
	DCB     0xFF, 0xFF, 0xFF, 0xFF
	DCB     0xFF, 0xFF, 0xFF, 0xFF
	DCB     0xFE, 0xCA, 0xFF, 0xFF	; disable FLASH security, 1:8 clock div

	EXPORT	__Vectors
	AREA    RESET, DATA, READONLY	; Vector Table mapped to address 0 at reset
__Vectors
	DCD     top_of_stack  		; Top of Stack
	DCD     Reset_Handler  		; Reset Handler
	DCD     Default_Handler 	; NMI Handler
	DCD     Default_Handler  	; Hard Fault Handler		
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     Default_Handler  	; SVCall Handler
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     Default_Handler  	; PendSV Handler	
	DCD     Default_Handler  	; SysTick Handler

	; External Interrupts
 	DCD     0  			; Reserved 
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved
	DCD     0  			; Reserved		
 	DCD     Default_Handler  	; FTFA command complete/read collision interrupt
 	DCD     Default_Handler  	; Low Voltage Detect, Low Voltage Warning
	DCD     0  			; Reserved
	DCD     I2C_ISR  		; I2C0 interrupt
 	DCD     I2C_ISR  		; I2C1 interrupt
 	DCD     SPI0_ISR  		; SPI0 interrupt
 	DCD     0  			; Reserved 
	DCD     Default_Handler  	; UART0 status/error interrupt
 	DCD     0  			; Reserved 
 	DCD     0  			; Reserved 
  	DCD     Default_Handler 	; ADC0 interrupt
 	DCD     Default_Handler  	; CMP0 interrupt
	DCD     Default_Handler  	; TPM0 fault, overflow and channels interrupt
  	DCD     Default_Handler  	; TPM1 fault, overflow and channels interrupt
 	DCD     0  			; Reserved
 	DCD     0  			; Reserved
  	DCD     0  			; Reserved
	DCD     0  			; Reserved
  	DCD     0  			; Reserved
  	DCD     0  			; Reserved
  	DCD     0  			; Reserved
  	DCD     0  			; Reserved
  	DCD     Default_Handler  	; MCG interrupt
  	DCD     LPTMR0_ISR  		; LPTimer interrupt
	DCD     0  			; Reserved 
  	DCD     Default_Handler  	; Port A interrupt
  	DCD     PORTB_IRQ  		; Port B interrupt

	EXPORT	Reset_Handler
	PRESERVE8
	THUMB				; use thumb instruction set	
	AREA    |.text|, CODE, READONLY
Reset_Handler	PROC
	IMPORT	start
	ENTRY	
	IOANDB	0xF3, SIM_COPC+SIM	; turn off COP (watchdog)
	ldr	R0, =start		; jump to main code
	bx 	R0
	ENDP

;--------------------------------------------------------------------
PORTB_IRQ	PROC
	PUTW	0x01A00103, PORT_PCR0+4*5, PORTB ; clear IF 	
	PUTW	0, 0, SCB_SCR		; disable VLPS/LSS sleep mode	
	bx	LR
	ENDP
		
;--------------------------------------------------------------------
LPTMR0_ISR	PROC			; low-power timer ISR
	IOORB	0xC1, LPTMR_CSR+LPTMR0 	; clear timer interrupt	
	PUTW	0, 0, SCB_SCR		; disable VLPS/LSS sleep mode
	bx	LR
	ENDP

;--------------------------------------------------------------------
	IMPORT	I2Cbuf
I2C_ISR		PROC
	ldrb	R1, [R0, #I2C_FLT]
	lsrs	R1, #7			; STOP condition detected?
	bcs	I2C_STOP		; YES - exit
	ldrb	R2, [R0, #I2C_S]	; get status byte
	strb	R2, [R0, #I2C_S]	; clear I2C interrupt flags
	ldrb	R1, [R0, #I2C_C1]	; get command byte
	lsrs	R1, #5			; TX ot RX mode?
	bcc	I2C_RX
	
I2C_TX	subs	R3, #1			; last byte transmitted?
	beq	tx1			; YES - check for receive
	lsrs	R2, #1			; NO  - is ACK received?		
	bcs	tx3			; NO  - generate STOP	
	ldrb	R1, [R7]		; YES - get next byte
	strb	R1, [R0, #I2C_D]	; and send it over
	add	R7, R10			; update data pointer	
	bx	LR

tx1	orrs	R4, R4			; receive bytes?
	beq	tx3			; NO - generate STOP
	movs	R1, #0xE0		; YES - switch to RX mode
	cmp	R4, #1			; receive only 1 byte?
	bne	tx2			; NO
	movs	R1, #0xE8		; YES - prepare to send NAK
tx2	strb	R1, [R0, #I2C_C1]	; switch to RX mode
	ldr	R7, =I2Cbuf
	ldrb	R1, [R0, #I2C_D]	; read dummy byte to start RX
	bx	LR
tx3	
	PUTB	0x60, I2C_FLT		; enable STOP interrupt
	PUTB	0x02, I2C_S
	PUTB	0xC0, I2C_C1		; generate STOP		
	bx	LR

I2C_RX	subs	R4, #1			; last byte read?
	bmi	rx3
	bne	rx1			; YES - exit
	PUTB	0x60, I2C_FLT		; enable STOP interrupt	
	PUTB	0x02, I2C_S
	PUTB	0xC0, I2C_C1		; generate STOP	
	b	rx2

rx1	cmp	R4, #1			; 2nd to last byte received?
	bne	rx2	
	PUTB	0xE8, I2C_C1		; YES - prepare to send NAK

rx2	ldrb	R1, [R0, #I2C_D]	; read received byte
	strb	R1, [R7]		; store it in buffer
	adds	R7, #1			; update buffer pointer
rx3	bx	LR	
	
I2C_STOP
	PUTB	0x40, I2C_FLT		; clear STOP flag
	PUTB	0x02, I2C_S
	PUTB	0x80, I2C_C1		; disable I2C interrupts
	ldr	R3, =SCB_SCR		; cancel sleep on exit 
	movs	R1, #0
	str	R1, [R3]
	bx	LR
	ENDP

;--------------------------------------------------------------------
	IMPORT	SPIbuf
SPI0_ISR	PROC
	ldrb	R1, [R0, #SPI_S]
	cmp	R3, #0 			; all bytes transmitted?
	bmi	spi3			; RX operation
	beq	spi1			; switch to RX
	ldrb	R1, [R7]		; R1 = next byte to transmit
	strb	R1, [R0, #SPI_D]	; load it to TX buffer
	adds	R7, #1			; update buffer pointer
	subs	R3, #1			; all bytes sent?
	beq	spi0			; YES - wait for transfer end
	bx	LR			; NO  - send more bytes
	
spi0	ldrb	R1, [R0, #SPI_D]	; clear RX flag (R3 = 0)
	PUTB	0xD0, SPI_C1		; enable RX interrupt
	movs	R2, #2
	bx	LR			
	
spi1	ldrb	R1, [R0, #SPI_D]	; wait for the end of TX
	subs	R2, #1
	beq	spi2
	bx	LR
	
spi2	cmp	R4, #0			; receive bytes?
	beq	spi4			; NO  - exit
	ldrb	R1, [R0, #SPI_D]	; YES - clear RX flag (R3 = 0)
	strb	R4, [R0, #SPI_D]	; and request the first byte	
	subs	R3, #1			; R3 becomes <0 now
	ldr	R7, =SPIbuf		; save receiver bytes there
	bx	LR

spi3	ldrb	R1, [R0, #SPI_D]	; RX mode: get byte from RX buffer
	strb	R1, [R7]		; load byte to SPI buffer
	adds	R7, #1			; update buffer pointer	
	subs	R4, #1			; all bytes received?
	beq	spi4			; YES - exit
	strb	R4, [R0, #SPI_D]	; NO  - request the next byte
	bx	LR			
	
spi4	ldr	R3, =SCB_SCR		; cancel sleep on exit 
	movs	R1, #0
	str	R1, [R3]
	PUTB	0x50, SPI_C1		; disable SPI interrupts	
	bx	LR
	ENDP		
		
;--------------------------------------------------------------------
Default_Handler	PROC			; default ISR placeholder
	b	.
	ENDP
	ENDP		
	ALIGN
	END
