	INCLUDE MKL02.inc		; CPU register definitions
	PRESERVE8
	THUMB				; use thumb instruction set
		
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
I2Cbuf	SPACE 	32			; I2C I/O buffer
SPIbuf	SPACE 	32			; SPI I/O buffer
Package	SPACE	16			; package	
	EXPORT	I2Cbuf
	EXPORT	SPIbuf
	EXPORT	Package		
				
	AREA    |.text|, CODE, READONLY	
MCG_setup	PROC			; switch to the FBI clock mode
	EXPORT	MCG_setup		
	PUTW	0x40000, SIM_CLKDIV1, SIM ; setup 0.8 MHz bus clock 1:5 divider
	PUTB	0, MCG_SC, MCG		; select 1:1 clock divider

	PUTB	0x44, MCG_C1		; set CLKS bit and preserve IREFS 
	movs	R2, #0x04		; wait for the CLKST bit is set  
	movs	R3, #0xFF		; max # of repetitions (for debugging)
mcg1	ldrb	R1, [R0, #MCG_S]	; start checking loop			
	tst	R1, R2			; check the CLKST bit
	bne	.+6
	subs	R3, #1
	bne	mcg1

;	PUTB	6, MCG_C4		; trim oscillator
	PUTB	0x07, MCG_C2		; select fast internal ref clock & LP mode
	movs	R2, #1			; wait for the IRCST bit is set 
	movs	R3, #0xFF		; max # of repetitions (for debugging)
mcg2	ldrb	R1, [R0, #MCG_S]
	tst	R1, R2			; check the IRCST bit
	bne	.+6
	subs	R3, #1
	bne	mcg2

	PUTB	0xA3, 0, OSC		; enable Xtal oscillator
	bx	LR
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
SMC_setup	PROC			; System Mode Controller configuration
	EXPORT	SMC_setup
	PUTB	0x20, SMC_PMPROT, SMC	; allow VLPR, VLPW, VLPS modes
	PUTB	0x42, SMC_PMCTRL	; enter VLPS mode in STOP	
	ldrb	R1, [R0, #SMC_PMSTAT]	; get current mode
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
PORTS_setup	PROC			; I/O Ports configuration
	EXPORT	PORTS_setup
	IOORH	0x0600, SIM_SCGC5+SIM 	; enable clock for I/O ports and LPTMR
	PUTW	0x00E00100, PORTA_GPCLR, PORTA	; configure PORTA for GPIO
	PUTW	0x00180000, PORTA_GPCLR		; diable pull-up on Xtal pins	
	PUTW	0x21, GPIOA_PDOR, GPIOA		; init port output
	PUTW	0x00FF, GPIOA_PDDR		; configure PORTA pins for output	
	PUTW	0x0003F0100, PORTB_GPCLR, PORTB ; configure PORTB for GPIO
	PUTW	0x000020103, PORTB_GPCLR	; enable pull-up on PB1	
	PUTW	0x18, GPIOB_PDOR, GPIOB		; init port output
	PUTW	0x0039, GPIOB_PDDR   		; configure PORTB pins for output
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
LPTMR0_setup	PROC			; Low-Power Timer configuration
	EXPORT	LPTMR0_setup
	IOORB	0x01, SIM_SCGC5+SIM 	; enable clock for LPTMR
	PUTB	0x3A, LPTMR0_PSR, LPTMR0; set OSC32 as clock source, 1:256 prescaler	
;	PUTW	128, LPTMR0_CMR 	; setup for 1 sec interval
;	PUTW	512, LPTMR0_CMR 	; setup for 4 sec interval
	PUTW	7860, LPTMR0_CMR 	; setup for 1 min interval
	PUTB	0xC1, LPTMR0_CSR	; start timer in autoreset, enable interrupt
	PUTW	0x10000000, 0, NVIC_ISER ; enable LPTMR0 interrupt in NVIC
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
I2C0_setup	PROC			; configuration of I2C module as master
	EXPORT	I2C0_setup
	PUTW	0x00180200, PORTB_GPCLR, PORTB	; set ALT2 function on I2C pins		
	IOORB	0x40, SIM_SCGC4+SIM	; enable clock for I2C module
	PUTB	0x00, I2C0_F, I2C0	; div by 20 I2C clock rate
	PUTW	1<<8, 0, NVIC_ISER	; enable I2C0 interrupt in NVIC
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
I2C0_transfer	PROC
	EXPORT	I2C0_transfer
	PUTW	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUTB	0xD0, I2C0_C1, I2C0	; enable I2C module TX, and interrupt 
	PUTB	0xF0, I2C0_C1		; generate START condition
	ldrb	R1, [R7]		; get first byte
	adds	R7, #1
	strb	R1, [R0, #I2C0_D]	; and send it over	
	wfi				; enter sleep mode	
        PUTB	0, I2C0_C1		; I2C reset	
	bx	LR
	ENDP

;-------------------------------------------------------------------------------
SPI0_setup	PROC
	EXPORT	SPI0_setup
	IOORW	0x400000, SIM_SCGC4+SIM	; enable clock for SPI module	
	PUTB	0x00, SPI0_BR, SPI0	; set bus_freq/2 baud rate
	PUTW	1<<10, 0, NVIC_ISER	; enable SPI0 interrupt in NVIC
	bx	LR
	ENDP

	ALIGN
	END
	