	INCLUDE MKL02.inc		; CPU register definitions
	EXPORT	start
	
	IMPORT	MCG_setup
	IMPORT	SMC_setup
	IMPORT	PORTS_setup
	IMPORT	LPTMR0_setup	
	IMPORT	I2C0_setup
	IMPORT	I2C0_transfer
	IMPORT	I2Cbuf
	IMPORT	SPI0_setup
	IMPORT	SPIbuf	
	IMPORT	SI4060_setup
	IMPORT	Send_SI4060
	IMPORT	Read_SI4060		
	IMPORT	Package
	
	PRESERVE8
	THUMB				; use thumb instruction set
	AREA    |.text|, CODE, READONLY
;----------------------------------------------------------------------------
start					; main user code
	bl	MCG_setup		; configure clock for VLPR mode
	bl	SMC_setup		; enable clocks to peripherals
	bl	PORTS_setup		; I/O ports setup
	bl	SPI0_setup		; setup SPI module
	bl	SI4060_setup		; init radio	
	bl	I2C0_setup		; setup I2C module
	bl	LPTMR0_setup		; start low-power timer	
	b	len

loop					; MAIN LOOP
	PUTW	SLEEPDEEP+SLEEPONEXIT, 0, SCB_SCR ; enable VLPS/LSS sleep mode
	wfi				; enter deep sleep mode
len	bl    	Read_Temp_Humi       	; read TEMP/HUMI from sensor R3=temp			
	bl	Send_Package 	
	b	loop
;	ENDP

;-----------------------------------------------------------------------
Send_Package	PROC
	push	{LR}
	mov	R0, R10
	subs	R0, #1			; time to measure battery?
	mov	R10, R0
	bne	sp1			; NO  - skip it
	movs	R0, #10			; YES - restore the counter
	mov	R10, R0
	ldr	R7, =Measure_Battery
	bl	Send_SI4060
	ldr	R7, =Read_CMD_BUFF
	movs	R4, #7			; response size
	bl	Read_SI4060
	subs	R7, #4
	ldrb	R0, [R7]		; get voltage value
	ldrb	R1, [R7, #1]
	rev16	R0, R0
	adds	R0, R1
	movs	R1, #3			; V*10 = 3*code/128
	muls	R0, R1, R0
	lsrs	R0, #7			; R0 = batt*10 (V)

sp1	ldr	R7, =Package		; compose package
	strb	R0, [R7, #6]		; add battery voltage
	mov	R0, R8
	strb	R0, [R7, #4]		; add humi value
	mov	R0, R9
	strb	R0, [R7, #5]		; add temp value
	bl	Send_SI4060		; load package in TX FIFO
		
	ldr	R7, =Start_TX	
	bl	Send_SI4060
	pop	{PC}
	ENDP

Read_CMD_BUFF				; read command response
	DCB	0x01, 0x44
Measure_Battery				; measure battery voltage
	DCB	0x03, 0x14, 0x08, 0xC9
Start_TX				; Start TX sequence
	DCB	0x05, 0x31, 0x00, 0x10, 0x00, 0x05
	ALIGN
;----------------------------------------------------------------------------
Read_Temp_Humi	PROC			; request TEMP/HUMI conversion
	push	{LR}
	ldr	R7, =I2Cbuf
	movs	R1, #0xF5		; request to measure humi 
	rev16	R1, R1
	adds	R1, #0x80		; Si7021 I2C address + write
	strh	R1, [R7]
	movs	R3, #2			; send 2 bytes
	movs	R4, #0			; receive 0 bytes
	bl	I2C0_transfer	

	PUTB	0xC0, LPTMR0_CSR, LPTMR0; disable LPTMR0
	PUTW	3, LPTMR0_CMR 		; setup for 25 msec interval	
	PUTB	0xC1, LPTMR0_CSR	; reenable LPTMR0
	PUTW	SLEEPDEEP, 0, SCB_SCR 	; enable VLPS/LSS sleep mode
	wfi				; wait for end of conversion
	PUTB	0xC0, LPTMR0_CSR, LPTMR0; disable LPTMR0
	PUTW	7860, LPTMR0_CMR 	; setup for 1 min interval	
	PUTB	0xC1, LPTMR0_CSR	; reenable LPTMR0

	ldr	R7, =I2Cbuf		; Read_Humi
	movs	R1, #0x81		; read humi  
	strb	R1, [R7]
	movs	R3, #1			; send 1 byte
	movs	R4, #2			; receive 2 bytes
	bl	I2C0_transfer
	subs	R7, #2
	ldrh	R1, [R7]		; R1 = humi reading
	rev16	R1, R1
	mov	R11, R1			; R11 = humi value
	movs	R0, #125
	muls	R0, R1, R0
	lsrs	R0, #16
	movs	R2, #1
	adcs	R0, R2
	subs	R0, #1
	mov	R8, R0			; R8 = humi

	ldr	R7, =I2Cbuf		; Read_Temp
	movs	R1, #0xE0		; request to read temp 
	rev16	R1, R1
	adds	R1, #0x80		; Si7021 I2C address + write
	strh	R1, [R7]
	movs	R3, #0			; send 2 bytes
	movs	R4, #0			; receive 0 bytes
	bl	I2C0_transfer

	ldr	R7, =I2Cbuf
	movs	R1, #0x81		; read temp  
	strb	R1, [R7]
	movs	R3, #1			; send 1 byte
	movs	R4, #2			; receive 2 bytes
	bl	I2C0_transfer
	subs	R7, #2
	ldrh	R1, [R7]		; R1 = temp reading
	rev16	R1, R1
	mov	R12, R1			; R12 = temp value
	movs	R0, #176
	muls	R0, R1, R0
	lsrs	R0, #16
	movs	R2, #0
	adcs	R0, R2
	subs	R0, #47
	mov	R9, R0			; R9 = temp
	
	pop	{PC}
	ENDP
	
	ALIGN 
	END
	