	INCLUDE MKL02.inc		; CPU register definitions
	IMPORT	SPIbuf
	IMPORT	Package	

CS	EQU	1<<5			; CS =  PA.5 (chip select)	
CTS	EQU	2			; CTS = PB.2 (clear to send)
	
	PRESERVE8
	THUMB				; use thumb instruction set
	AREA    |.text|, CODE, READONLY

;-----------------------------------------------------------------
SI4060_setup	PROC
	EXPORT	SI4060_setup
	push	{LR}
	ldr	R5, =GPIOA		; pointer to GPIO
	ldr	R6, =GPIOB
	movs	R0, #10
	mov	R10, R0			; battery checking period
	
	movs	R0, #0x80		; power-up delay
	lsls	R0, #2
	subs	R0, #1
	nop
	bne	.-4	
	
	ldr	R7, =SI4060_Config
	ldrb	R2, [R7]		; get # of commands to send
	adds	R7, #1
s4s	push	{R2}
	bl	Send_SI4060
	pop	{R2}
	subs	R2, #1
	bne	s4s
	
	ldr	R7, =SI4060_sleep	; place radio in sleep	
	bl	Send_SI4060		
	
	ldr	R7, =Package		; copy package template from
	ldr	R1, =Package_Flash	; Flash to RAM
	ldr	R0, [R1]
	str	R0, [R7]
	ldr	R0, [R1, #4]
	str	R0, [R7, #4]
	pop	{PC}
	ENDP
		
	ALIGN
Package_Flash
	DCB	0x06, 0x66, 0x04, 0xAA, 0x41, 0x42, 0x43, 0x00
;-----------------------------------------------------------------
Send_SI4060	PROC			; send command/data/property to radio
	EXPORT	Send_SI4060
	movs	R4, #0	
Read_SI4060	
	EXPORT	Read_SI4060
	PUTW	0x00C00300, PORTA_GPCLR, PORTA	; connect pins to SPI 
	PUTW	0x00010300, PORTB_GPCLR, PORTB
wcts1	ldr	R3, [R6, #GPIOB_PDIR]	; check CTS flag
	lsrs	R3, #(CTS+1)
	bcc	wcts1			; wait for CTS

	ldrb	R3, [R7]		; R3 = #bytes to TX
	adds	R7, #1			
	PUTW	SLEEPONEXIT, 0, SCB_SCR ; set sleep on exit
	PUTB	0x50, SPI0_C1, SPI0	; enable SPI module TX, and interrupt
	ldr	R2, =GPIOA
	movs	R1, #CS
	str	R1, [R2, #GPIOA_PCOR]	; CS down
	PUTB	0x70, SPI0_C1		; enable SPI TX empty interrupt	
	wfi	
	ldr	R2, =GPIOA
	movs	R1, #CS
	str	R1, [R2, #GPIOA_PSOR]	; CS up
	PUTB	0, SPI0_C1		; SPI reset
	PUTW	0x00C00100, PORTA_GPCLR, PORTA	; disconnect pins from SPI 
	PUTW	0x00010100, PORTB_GPCLR, PORTB	
	bx	LR
	ENDP
		
;-------------------------------------------------------------------	
	ALIGN
SI4060_Config ; 903MHz, 1.2kbps, 5.15KHz,  5 bytes payload + CRC16, 4 bytes pre, D3 91 synch
        DCB     0x1A    ; config data size
	DCB     0x07, 0x02, 0x01, 0x00, 0x01, 0xC9, 0xC3, 0x80		; RF_POWER_UP
        DCB     0x08, 0x13, 0x01, 0x08, 0x01, 0x01, 0x01, 0x0B, 0x00	; GPIO_PIN_CFG: int. disabled
        DCB     0x06, 0x11, 0x00, 0x02, 0x00, 0x52, 0x00		; GLOBAL_XO_TUNE
        DCB     0x05, 0x11, 0x00, 0x01, 0x03, 0x60 			; GLOBAL_CONFIG
        DCB     0x05, 0x11, 0x01, 0x01, 0x00, 0x00			; INT_CTL_ENABLE int disable
        DCB     0x08, 0x11, 0x02, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00	; FRR_CTL_A_MODE
        DCB     0x07, 0x11, 0x10, 0x03, 0x00, 0x04, 0x14, 0x00		; PREAMBLE_TX_LENGTH
        DCB     0x09, 0x11, 0x10, 0x05, 0x04, 0x31, 0x00, 0x00, 0x00, 0x00	; PREAMBLE_CONFIG
        DCB     0x09, 0x11, 0x11, 0x05, 0x00, 0x01, 0xCB, 0x89, 0x00, 0x00 	; SYNC_CONFIG (2 bytes 0xD3 0x91)
        DCB     0x05, 0x11, 0x12, 0x01, 0x00, 0x84			; PKT_CRC_CONFIG
        DCB     0x08, 0x11, 0x12, 0x04, 0x03, 0xFF, 0xFF, 0x00, 0x02	; PKT_CRC_CONFIG
        DCB     0x05, 0x11, 0x12, 0x01, 0x0B, 0x30			; PKT_TX_THRESHOLD
        DCB     0x10, 0x11, 0x12, 0x0C, 0x0D, 0x00, 0x05, 0x04, 0xA2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        DCB     0x0C, 0x11, 0x12, 0x08, 0x19, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ; RF_PKT_FIELD_4_LENGTH_12_8_8
        DCB     0x10, 0x11, 0x20, 0x0C, 0x00, 0x02, 0x00, 0x07, 0x00, 0x2E, 0xE0, 0x01, 0xC9, 0xC3, 0x80, 0x00, 0x00
        DCB     0x05, 0x11, 0x20, 0x01, 0x0C, 0xB4			; MODEM_FREQ_DEV (5.15 KHz)
        DCB     0x0C, 0x11, 0x20, 0x08, 0x18, 0x01, 0x80, 0x08, 0x03, 0xC0, 0x00, 0x24, 0x20
        DCB     0x0D, 0x11, 0x20, 0x09, 0x22, 0x03, 0x0D, 0x00, 0xA7, 0xC6, 0x00, 0x54, 0x02, 0xC2 ; 903 MHz
        DCB     0x0B, 0x11, 0x20, 0x07, 0x2C, 0x04, 0x36, 0x80, 0x01, 0x5D, 0x07, 0x80
        DCB     0x05, 0x11, 0x20, 0x01, 0x35, 0xE2
        DCB     0x0D, 0x11, 0x20, 0x09, 0x38, 0x11, 0xAB, 0xAB, 0x00, 0x02, 0xFF, 0xFF, 0x00, 0x2B
        DCB     0x0A, 0x11, 0x20, 0x06, 0x42, 0xA4, 0x02, 0xD6, 0x81, 0x01, 0x29
        DCB     0x05, 0x11, 0x20, 0x01, 0x51, 0x08			; RF_MODEM_CLKGEN_BAND
        DCB     0x08, 0x11, 0x22, 0x04, 0x00, 0x18, 0x19, 0xC0, 0x3D	; RF_PA_MODE
        DCB     0x0B, 0x11, 0x23, 0x07, 0x00, 0x2C, 0x0E, 0x0B, 0x04, 0x0C, 0x73, 0x03
        DCB     0x0B, 0x11, 0x40, 0x07, 0x00, 0x3B, 0x09, 0x99, 0x99, 0x22, 0x22, 0x20

SI4060_sleep	; send to sleep sequence
	DCB	0x02, 0x34, 0x01

	ALIGN
	END	